from __future__ import annotations

import re
import typing
import quart

from .app import app

_VALID_LANGUAGE_NAMES = re.compile(r'^[-a-z0-9_]+$')


def parse_accept_language(value: str) -> typing.List[str]:
    value = value.strip()
    if not value or value == '*':
        return []
    tuples = []
    for entry in value.split(','):
        attrs = entry.split(';')
        name = attrs.pop(0).strip().lower()
        q = 1.0
        for attr in attrs:
            if not '=' in attr: continue
            key, value = attr.split('=', maxsplit=1)
            if key.strip().lower() == 'q':
                try:
                    q = float(value.strip())
                except ValueError:
                    q = 0.0
        if q >= 0.0:
            tuples.append((q, name))
    tuples.sort()
    have = set()
    result = []
    for (_q, name) in tuples:
        if name in have: continue
        if name == '*': break
        have.add(name)
        if _VALID_LANGUAGE_NAMES.match(name):
            result.append(name)
        short_name = name.split('-', maxsplit=1)[0].split('_', maxsplit=1)[0]
        if not short_name or short_name in have: continue
        have.add(short_name)
        result.append(short_name)
    return result


@app.before_request
def detect_language():
    g = quart.g
    r = quart.request
    s = quart.session
    lang = r.args.get('setlang')
    if lang:
        lang = lang.strip().lower()
        if lang and _VALID_LANGUAGE_NAMES.match(lang):
            s = quart.session
            s['lang'] = lang
            g.langs = [lang]
            return
    lang = s.get('lang')
    if lang:
        lang = lang.strip().lower()
        if lang and _VALID_LANGUAGE_NAMES.match(lang):
            g.langs = [lang]
            return
    acc_lang = ','.join(r.headers.get_all('Accept-Language'))
    g.langs = parse_accept_language(acc_lang)
